@@ -1,20 +1,10 @@
-/*-*- c++ -*-******************************************************************
- * Qwt Widget Library 
+/* -*- mode: C++ ; c-file-style: "stroustrup" -*- *****************************
+ * Qwt Widget Library
  * Copyright (C) 1997   Josef Wilgen
+ * Copyright (C) 2002   Uwe Rathmann
  * 
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * modify it under the terms of the Qwt License, Version 1.0
  *****************************************************************************/
 
 #ifndef QWT_RECT_H
@@ -23,6 +13,9 @@
 #include <qrect.h>
 #include "qwt_global.h"
 
+class QPoint;
+class QPointArray;
+
 /*!
   \brief Some extensions for QRect
 */
@@ -38,6 +31,16 @@ public:
     QRect cutLeft(int w, int distLeft = 0, int distRight = 0);
     QRect cutRight(int w, int distLeft = 0, int distRight = 0);
     const QwtRect &cutMargin(int mLeft, int mRight, int Top, int mBottom);
+
+    QPointArray clip(const QPointArray &) const;
+
+private:
+    enum Edge { Left, Top, Right, Bottom, NEdges };
+
+    void clipEdge(Edge, const QPointArray &, QPointArray &) const;
+    bool insideEdge(const QPoint &, Edge edge) const;
+    QPoint intersectEdge(const QPoint &p1, 
+        const QPoint &p2, Edge edge) const;
 };
 
 #endif
