@@ -1,20 +1,10 @@
-/*-*- c++ -*-******************************************************************
- * Qwt Widget Library 
+/* -*- mode: C++ ; c-file-style: "stroustrup" -*- *****************************
+ * Qwt Widget Library
  * Copyright (C) 1997   Josef Wilgen
+ * Copyright (C) 2002   Uwe Rathmann
  * 
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- * 
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * modify it under the terms of the Qwt License, Version 1.0
  *****************************************************************************/
 
 #include "qwt_math.h"
@@ -126,6 +116,24 @@ void QwtAutoScale::adjust(double *x, int num, int reset)
 }
 
 
+/*!
+  \brief Adjust the scale to include a given array of input values.
+
+  This member function extends the boundaries of the scale
+  and re-calculates the step size if necessary in order
+  to include all values in the array. If the reset
+  parameter has nonzero value, the previous state will
+  be cleared.
+  
+  \param x QwtArray<double> of input values
+  \param reset If != 0 reset the scale's contents
+*/
+void QwtAutoScale::adjust(const QwtArray<double> &x, int reset)
+{
+    adjust(x.data(), x.size(), reset);
+}
+
+
 /*!
   \brief Adjust the scale to include a specified interval
 
@@ -179,8 +187,22 @@ void QwtAutoScale::build()
     }
     else
     {
-       d_scldiv.rebuild(d_scaleMin, d_scaleMax, d_maxMajor, d_maxMinor,
-            bool(d_scaleOpt & Logarithmic), d_step, FALSE); 
+        double start, stop, step;
+        if (d_scaleOpt & Inverted)
+        {
+            start = d_scaleMax;
+            stop = d_scaleMin;
+            step = -d_step;
+        }
+        else
+        {
+            start = d_scaleMin;
+            stop = d_scaleMax;
+            step = d_step;
+        }
+
+        d_scldiv.rebuild(start, stop, d_maxMajor, d_maxMinor,
+            bool(d_scaleOpt & Logarithmic), step, FALSE); 
     }
 }
 
@@ -302,7 +324,7 @@ void QwtAutoScale::buildLogScale ()
             minval = d_lref;
     }
 
-	const double ticks = (d_maxMajor > 0) ? double(d_maxMajor) : 1;
+    const double ticks = (d_maxMajor > 0) ? double(d_maxMajor) : 1;
 
     setRange(minval, maxval);
 
@@ -314,7 +336,7 @@ void QwtAutoScale::buildLogScale ()
     double step;
     if ((decades > 1.0) && (decades > ticks))
     {
-		double ipart;
+        double ipart;
         // One interval contains more than one decade.
         // The number of decades in an interval is adjusted
         // to be a multiple of 2,3,5, or 10.
@@ -330,7 +352,7 @@ void QwtAutoScale::buildLogScale ()
         else
            fpart = 10.0;
 
-        step = pow (10, ipart) * fpart;
+        step = pow (10.0, ipart) * fpart;
 
     }
     else                // The minimal step size is one decade.
@@ -340,9 +362,9 @@ void QwtAutoScale::buildLogScale ()
     
     if (!(d_scaleOpt & Floating))
     {
-        d_scaleMin = pow (10, step * 
+        d_scaleMin = pow (10.0, step * 
             floor ((log10(d_scaleMin) + MinEps * step) / step));
-        d_scaleMax = pow (10, step * 
+        d_scaleMax = pow (10.0, step * 
             ceil ((log10(d_scaleMax) - MinEps * step) / step));
     }
 
@@ -472,7 +494,18 @@ void QwtAutoScale::setMaxMinor(int mx)
 
 
 /*!
-  \todo Documentation
+  \brief Specify a user-defined range
+
+  \param x1 user-defined lower boundary
+  \param x2 user-defined upper boundary
+
+  \warning
+  \li if x1 > x2, the lower boundary will be set to x2 and the upper boundary
+      to x1.
+  \li if x1 == x2, the auto-scaler sets the boundaries to (-0.5, 0.5) for
+      linear scales and to (LOG_MIN, LOG_MAX) for logarithmic scales.
+
+  \sa QwtAutoScale::setScale(), LOG_MIN, LOG_MAX.
 */
 void QwtAutoScale::setRange(double x1, double x2)
 {
@@ -482,7 +515,7 @@ void QwtAutoScale::setRange(double x1, double x2)
     if (d_scaleOpt & Logarithmic)
     {
         minval = qwtMin(qwtMax(minval, LOG_MIN), LOG_MAX);
-        maxval = qwtMin(qwtMax(maxval,LOG_MIN), LOG_MAX);
+        maxval = qwtMin(qwtMax(maxval, LOG_MIN), LOG_MAX);
     }
     
     double delta = maxval - minval;
@@ -531,11 +564,14 @@ void QwtAutoScale::setRange(double x1, double x2)
   previous values.
 
   \warning
-  \li if xmin > xmax, xmax will be the lower boundary.
-  \li if xmin == xmax, the auto-scaler sets the boundaries to (-0.5, 0.5).
+  \li if xmin > xmax, the lower boundary will be set to xmax and the upper
+      boundary to xmin.
+  \li if xmin == xmax, the auto-scaler sets the boundaries to (-0.5, 0.5)
+      for linear scales and to (LOG_MIN, LOG_MAX) for logarithmic scales.
   \li Options and margins have no effect while auto-scaling is switched off.
 
-  \sa QwtAutoScale::setMaxMajor(), QwtAutoScale::setAutoScale()
+  \sa QwtAutoScale::setMaxMajor(), QwtAutoScale::setAutoScale(),
+      LOG_MIN, LOG_MAX.
 */
 void QwtAutoScale::setScale(double xmin, double xmax, double step)
 {
@@ -627,8 +663,9 @@ void QwtAutoScale::setOptions(int opt)
 
   \warning The reference value for logarithmic scales is limited to
   ( LOG_MIN / 2 <= reference <= LOG_MAX /2 ). If the specified
-  value is less than LOG_MIN (defined in qwt_math.h), it will
-  be set to 1.0 for logarithmic scales.
+  value is less than LOG_MIN, it will be set to 1.0 for logarithmic scales.
+
+  \sa LOG_MIN, LOG_MAX.
 */
 void QwtAutoScale::setReference(double r)
 {
@@ -687,7 +724,7 @@ const QwtScaleDiv &QwtAutoScale::scaleDiv() const
 
 /*!
     If true, rebuild scale automatically with call to 'adjust'
-    \sa QwtAutoScale::autoRebuild
+    \sa QwtAutoScale::autoRebuild()
 */
 
 void QwtAutoScale::setAutoRebuild(bool tf) 
@@ -697,7 +734,7 @@ void QwtAutoScale::setAutoRebuild(bool tf)
 
 /*!
     \return If true, rebuild scale automatically with call to 'adjust'
-    \sa QwtAutoScale::setAutoRebuild
+    \sa QwtAutoScale::setAutoRebuild()
 */
 bool QwtAutoScale::autoRebuild() const
 {
