@@ -1,36 +1,24 @@
-/*-*- c++ -*-******************************************************************
+/* -*- mode: C++ ; c-file-style: "stroustrup" -*- *****************************
  * Qwt Widget Library
  * Copyright (C) 1997   Josef Wilgen
+ * Copyright (C) 2002   Uwe Rathmann
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * modify it under the terms of the Qwt License, Version 1.0
  *****************************************************************************/
 
 #include <qpainter.h>
+#include <qevent.h>
 #include <qstyle.h>
 #include <qpixmap.h>
 #include <qdrawutil.h>
 #include "qwt_math.h"
+#include "qwt_paint_buffer.h"
 #include "qwt_thermo.h"
 
-//------------------------------------------------------------
-//.C MEMBER FUNCTION DESCRIPTION
-//------------------------------------------------------------
-
 //! Constructor
 QwtThermo::QwtThermo(QWidget *parent, const char *name): 
-    QWidget(parent, name, WRepaintNoErase|WResizeNoErase)
+    QWidget(parent, name, Qt::WRepaintNoErase|Qt::WResizeNoErase)
 {
     init();
 }
@@ -40,7 +28,7 @@ QwtThermo::~QwtThermo()
 {
 }
 
-//! initialize data members
+//! Initialize the data members.
 void QwtThermo::init()
 {
     // initialize data members
@@ -55,17 +43,18 @@ void QwtThermo::init()
     d_alarmLevel = 0.0;
     d_alarmEnabled = 0;
 
-    // initialize colors;
-    d_fillColor = black;
-    d_alarmColor = white;
+    // initialize brushes;
+    d_fillBrush = QBrush(Qt::black);
+    d_alarmBrush = QBrush(Qt::white);
 
     // initialize scales
     d_map.setDblRange(d_minValue, d_maxValue);
-    d_scale.setScale(d_minValue, d_maxValue, d_maxMajor, d_maxMinor);
+    scaleDraw()->setScale(d_minValue, d_maxValue,
+        scaleMaxMajor(), scaleMaxMinor());
 }
 
 
-//! Set the current value
+//! Set the current value.
 void QwtThermo::setValue(double v)
 {
     if (d_value != v)
@@ -75,30 +64,25 @@ void QwtThermo::setValue(double v)
     }
 }
 
-//! Qt paint event
+//! Qt paint event.
 void QwtThermo::paintEvent(QPaintEvent *e)
 {
     // Use double-buffering
-    QRect ur = e->rect();
+    const QRect &ur = e->rect();
     if ( ur.isValid() )
     {
-        QPixmap pix( ur.size() );
-        pix.fill( this, ur.topLeft() );
-        QPainter p( &pix, this );
-        p.translate( -ur.x(), -ur.y() );
-        draw(&p, ur);
-        p.end();
-        bitBlt( this, ur.topLeft(), &pix );
+        QwtPaintBuffer paintBuffer(this, ur);
+        draw(paintBuffer.painter(), ur);
     }
 }
 
-//! redraw the thermo
+//! Draw the whole QwtThermo.
 void QwtThermo::draw(QPainter *p, const QRect& ur)
 {
     if ( !d_thermoRect.contains(ur) )
     {
         if (d_scalePos != None)
-            d_scale.draw(p);
+            scaleDraw()->draw(p);
 
         qDrawShadePanel(p,
             d_thermoRect.x() - d_borderWidth,
@@ -116,94 +100,98 @@ void QwtThermo::resizeEvent(QResizeEvent *)
     layoutThermo( FALSE );
 }
 
-//! Recalculate the thermo's geometry and layout based on
-//  the current rect and fonts.
-//  \param update_geometry notify the layout system and call update
-//         to redraw the scale
+/*!
+  Recalculate the QwtThermo geometry and layout based on
+  the QwtThermo::rect() and the fonts.
+  \param update_geometry notify the layout system and call update
+         to redraw the scale
+*/
 void QwtThermo::layoutThermo( bool update_geometry )
 {
-    QRect r = this->rect();
+    QRect r = rect();
     int mbd = 0;
     if ( d_scalePos != None )
-        mbd = d_scale.minBorderDist( fontMetrics() );
+    {
+        int d1, d2;
+        scaleDraw()->minBorderDist(fontMetrics(), d1, d2);
+        mbd = QMAX(d1, d2);
+    }
 
     if ( d_orient == Qt::Horizontal )
     {
         switch ( d_scalePos )
         {
-            case None:
-                d_thermoRect.setRect(r.x() + d_borderWidth,
-                    r.y() + d_borderWidth,
-                    r.width() - 2*d_borderWidth,
-                    r.height() - 2*d_borderWidth);
+            case Top:
+                d_thermoRect.setRect(
+                    r.x() + mbd + d_borderWidth,
+                    r.y() + r.height()
+                    - d_thermoWidth - 2*d_borderWidth,
+                    r.width() - 2*(d_borderWidth + mbd),
+                    d_thermoWidth);
+                scaleDraw()->setGeometry(
+                    d_thermoRect.x(),
+                    d_thermoRect.y() - d_borderWidth - d_scaleDist,
+                    d_thermoRect.width(),
+                    QwtScaleDraw::Top);
                 break;
 
             case Bottom:
-                d_thermoRect.setRect(r.x() + mbd + d_borderWidth,
+            case None: // like Bottom but without scale
+            default:   // inconsistent orientation and scale position
+                       // Mapping between values and pixels requires
+                       // initialization of the scale geometry
+                d_thermoRect.setRect(
+                    r.x() + mbd + d_borderWidth,
                     r.y() + d_borderWidth,
                     r.width() - 2*(d_borderWidth + mbd),
                     d_thermoWidth);
-                d_scale.setGeometry(d_thermoRect.x(),
+                scaleDraw()->setGeometry(
+                    d_thermoRect.x(),
                     d_thermoRect.y() + d_thermoRect.height()
                     + d_borderWidth + d_scaleDist,
                     d_thermoRect.width(),
                     QwtScaleDraw::Bottom);
                 break;
-
-            case Top:
-            default:
-                d_thermoRect.setRect(r.x() + mbd + d_borderWidth,
-                    r.y() + r.height()
-                    - d_thermoWidth - 2*d_borderWidth,
-                    r.width() - 2*(d_borderWidth + mbd),
-                    d_thermoWidth);
-                d_scale.setGeometry(d_thermoRect.x(),
-                    d_thermoRect.y() - d_borderWidth - d_scaleDist,
-                    d_thermoRect.width(),
-                    QwtScaleDraw::Top);
-                break;
         }
         d_map.setIntRange(d_thermoRect.x(),
-        d_thermoRect.x() + d_thermoRect.width() - 1);
+            d_thermoRect.x() + d_thermoRect.width() - 1);
     }
     else // Qt::Vertical
     {
         switch ( d_scalePos )
         {
-            case None:
-                d_thermoRect.setRect(r.x() + d_borderWidth,
-                    r.y() + d_borderWidth,
-                    r.width() - 2*d_borderWidth,
-                    r.height() - 2*d_borderWidth);
-                break;
-
-            case Left:
-                d_thermoRect.setRect(r.x() +  r.width()
-                    - 2*d_borderWidth - d_thermoWidth,
-                    r.y() + mbd + d_borderWidth,
-                    d_thermoWidth,
-                    r.height() - 2*(d_borderWidth + mbd));
-                d_scale.setGeometry(d_thermoRect.x() - 
-                    d_scaleDist - d_borderWidth,
-                    d_thermoRect.y(),
-                    d_thermoRect.height(),
-                    QwtScaleDraw::Left);
-                break;
-
             case Right:
-            default:
-                d_thermoRect.setRect(r.x() + d_borderWidth,
+                d_thermoRect.setRect(
+                    r.x() + d_borderWidth,
                     r.y() + mbd + d_borderWidth,
                     d_thermoWidth,
                     r.height() - 2*(d_borderWidth + mbd));
-                d_scale.setGeometry(d_thermoRect.x() + d_thermoRect.width()
+                scaleDraw()->setGeometry(
+                    d_thermoRect.x() + d_thermoRect.width()
                     + d_borderWidth + d_scaleDist,
                     d_thermoRect.y(),
                     d_thermoRect.height(),
                     QwtScaleDraw::Right);
                 break;
+
+            case Left:
+            case None: // like Left but without scale
+            default:   // inconsistent orientation and scale position
+                       // Mapping between values and pixels requires
+                       // initialization of the scale geometry
+                d_thermoRect.setRect(
+                    r.x() + r.width() - 2*d_borderWidth - d_thermoWidth,
+                    r.y() + mbd + d_borderWidth,
+                    d_thermoWidth,
+                    r.height() - 2*(d_borderWidth + mbd));
+                scaleDraw()->setGeometry(
+                    d_thermoRect.x() - d_scaleDist - d_borderWidth,
+                    d_thermoRect.y(),
+                    d_thermoRect.height(),
+                    QwtScaleDraw::Left);
+                break;
         }
-        d_map.setIntRange( d_thermoRect.y() + d_thermoRect.height() - 1,
+        d_map.setIntRange(d_thermoRect.y() + d_thermoRect.height() - 1,
             d_thermoRect.y());
     }
     if ( update_geometry )
@@ -214,16 +202,20 @@ void QwtThermo::layoutThermo( bool update_geometry )
 }
 
 /*!
-  \brief Change the thermometer's orientation
+  \brief Set the thermometer orientation and the scale position.
 
   The scale position None disables the scale.
   \param o orientation. Possible values are Qt::Horizontal and Qt::Vertical.
          The default value is Qt::Vertical.
-  \param s Position of the scale. For a horizontal
-         orientation, the scale position can
-     be Top, Bottom or None. A vertical
-     thermometer may have the scale positions
-     Left, Right or None. The default is None.
+  \param s Position of the scale.
+         The default value is None.
+
+  A valid combination of scale position and orientation is enforced:
+  - a horizontal thermometer can have the scale positions Top, Bottom or None;
+  - a vertical thermometer can have the scale positions Left, Right or None;
+  - an invalid scale position will default to None.
+
+  \sa QwtThermo::setScalePosition()
 */
 void QwtThermo::setOrientation(Qt::Orientation o, ScalePos s)
 {
@@ -248,21 +240,51 @@ void QwtThermo::setOrientation(Qt::Orientation o, ScalePos s)
     layoutThermo();
 }
 
-//! notify changed font
+/*!
+  \brief Change the scale position (and thermometer orientation).
+
+  \param s Position of the scale.
+  
+  A valid combination of scale position and orientation is enforced:
+  - if the new scale position is Left or Right, the scale orientation will
+    become Qt::Vertical;
+  - if the new scale position is Bottom or Top, the scale orientation will
+    become Qt::Horizontal;
+  - if the new scale position is None, the scale orientation will not change.
+
+  \sa QwtThermo::setOrientation()
+*/
+void QwtThermo::setScalePosition(ScalePos s)
+{
+    if ((s == Bottom) || (s == Top))
+        setOrientation(Qt::Horizontal, s);
+    else if ((s == Left) || (s == Right))
+        setOrientation(Qt::Vertical, s);
+    else
+        setOrientation(d_orient, None);
+}
+
+//! Return the scale position.
+QwtThermo::ScalePos QwtThermo::scalePosition() const
+{
+    return d_scalePos;
+}
+
+//! Notify a font change.
 void QwtThermo::fontChange(const QFont &f)
 {
     QWidget::fontChange( f );
     layoutThermo();
 }
 
-//!notify changed scale
+//! Notify a scale change.
 void QwtThermo::scaleChange()
 {
     update();
     layoutThermo();
 }
 
-//! Redraw the thermometer
+//! Redraw the liquid in thermometer pipe.
 void QwtThermo::drawThermo(QPainter *p)
 {
     int alarm  = 0, taval = 0;
@@ -415,9 +437,9 @@ void QwtThermo::drawThermo(QPainter *p)
     p->fillRect(bRect, colorGroup().color(QColorGroup::Background));
 
     if (alarm)
-       p->fillRect(aRect, d_alarmColor);
+       p->fillRect(aRect, d_alarmBrush);
 
-    p->fillRect(fRect, d_fillColor);
+    p->fillRect(fRect, d_fillBrush);
 }
 
 //! Set the border width of the pipe.
@@ -431,49 +453,101 @@ void QwtThermo::setBorderWidth(int w)
     }
 }
 
+//! Return the border width of the thermometer pipe.
+int QwtThermo::borderWidth() const
+{
+    return d_borderWidth;
+}
+
 /*!
   \brief Set the range
-  \param vminValue corresponding lower or left end of the thermometer
-  \param vmax Value corresponding to the upper or right end of the thermometer
+  \param vmin value corresponding lower or left end of the thermometer
+  \param vmax value corresponding to the upper or right end of the thermometer
+  \param logarithmic logarithmic mapping, TRUE or FALSE 
 */
-void QwtThermo::setRange(double vmin, double vmax)
+void QwtThermo::setRange(double vmin, double vmax, bool logarithmic)
 {
     d_minValue = vmin;
     d_maxValue = vmax;
 
-    d_map.setDblRange(d_minValue, d_maxValue);
+    d_map.setDblRange(d_minValue, d_maxValue, logarithmic);
     if (!hasUserScale())
     {
-        QwtScaleDiv oldscl(d_scale.scaleDiv());
+        QwtScaleDiv oldscl(scaleDraw()->scaleDiv());
 
-        d_scale.setScale(d_minValue, d_maxValue, d_maxMajor, d_maxMinor);
-        if (oldscl != d_scale.scaleDiv())
+        scaleDraw()->setScale(d_minValue, d_maxValue,
+            scaleMaxMajor(), scaleMaxMinor(), 0.0, logarithmic);
+        if (oldscl != scaleDraw()->scaleDiv())
             scaleChange();
     }
     layoutThermo();
 }
 
+/*!
+  \brief Change the brush of the liquid.
+  \param c New brush. The default brush is solid black.
+*/
+void QwtThermo::setFillBrush(const QBrush& b)
+{
+    d_fillBrush = b;
+    update();
+}
+
+//! Return the liquid brush.
+const QBrush& QwtThermo::fillBrush() const
+{
+    return d_fillBrush;
+}
+
 /*!
   \brief Change the color of the liquid.
   \param c New color. The default color is black.
 */
 void QwtThermo::setFillColor(const QColor &c)
 {
-    d_fillColor = c;
+    d_fillBrush.setColor(c);
     update();
 }
 
+//! Return the liquid color.
+const QColor &QwtThermo::fillColor() const
+{
+    return d_fillBrush.color();
+}
+
 /*!
-  \brief Specify liquid color above the alarm threshold
+  \brief Specify the liquid brush above the alarm threshold
+  \param c New brush. The default is solid white.
+*/
+void QwtThermo::setAlarmBrush(const QBrush& b)
+{
+    d_alarmBrush = b;
+    update();
+}
+
+//! Return the liquid brush above the alarm threshold.
+const QBrush& QwtThermo::alarmBrush() const
+{
+    return d_alarmBrush;
+}
+
+/*!
+  \brief Specify the liquid color above the alarm threshold
   \param c New color. The default is white.
 */
 void QwtThermo::setAlarmColor(const QColor &c)
 {
-    d_alarmColor = c;
+    d_alarmBrush.setColor(c);
     update();
 }
 
-//! Specify the alarm threshold
+//! Return the liquid color above the alarm threshold.
+const QColor &QwtThermo::alarmColor() const
+{
+    return d_alarmBrush.color();
+}
+
+//! Specify the alarm threshold.
 void QwtThermo::setAlarmLevel(double v)
 {
     d_alarmLevel = v;
@@ -481,7 +555,13 @@ void QwtThermo::setAlarmLevel(double v)
     update();
 }
 
-//! Change the width of the pipe
+//! Return the alarm threshold.
+double QwtThermo::alarmLevel() const
+{
+    return d_alarmLevel;
+}
+
+//! Change the width of the pipe.
 void QwtThermo::setPipeWidth(int w)
 {
     if (w > 0)
@@ -491,6 +571,12 @@ void QwtThermo::setPipeWidth(int w)
     }
 }
 
+//! Return the width of the pipe.
+int QwtThermo::pipeWidth() const
+{
+    return d_thermoWidth;
+}
+
 
 /*!
   \brief Specify the distance between the pipe's endpoints
@@ -512,15 +598,21 @@ void QwtThermo::setMargin(int)
 
 
 /*!
-  \brief Enable or disable alarm threshold
-  \param tf \c ZERO (disabled) or non-zero (enabled)
+  \brief Enable or disable the alarm threshold
+  \param tf TRUE (disabled) or FALSE (enabled)
 */
-void QwtThermo::setAlarmEnabled(int tf)
+void QwtThermo::setAlarmEnabled(bool tf)
 {
     d_alarmEnabled = tf;
     update();
 }
 
+//! Return if the alarm threshold is enabled or disabled.
+bool QwtThermo::alarmEnabled() const
+{
+    return d_alarmEnabled;
+}
+
 /*!
   \return Fixed/MinimumExpanding for vertical, 
           MinimumExpanding/Fixed for horizontal thermos.
@@ -529,8 +621,8 @@ void QwtThermo::setAlarmEnabled(int tf)
 QSizePolicy QwtThermo::sizePolicy() const
 {
     QSizePolicy sp;
-    if ( d_scale.orientation() == QwtScaleDraw::Left ||
-        d_scale.orientation() == QwtScaleDraw::Right )
+    if ( scaleDraw()->orientation() == QwtScaleDraw::Left ||
+        scaleDraw()->orientation() == QwtScaleDraw::Right )
     {
         sp.setHorData( QSizePolicy::Fixed );
         sp.setVerData( QSizePolicy::MinimumExpanding );
@@ -563,8 +655,8 @@ QSize QwtThermo::minimumSizeHint() const
 
     if ( d_scalePos != None )
     {
-        int smw = d_scale.minWidth( QPen(), fontMetrics() );
-        int smh = d_scale.minHeight( QPen(), fontMetrics() );
+        int smw = scaleDraw()->minWidth( QPen(), fontMetrics() );
+        int smh = scaleDraw()->minHeight( QPen(), fontMetrics() );
 
         if ( d_orient == Qt::Vertical )
         {
@@ -593,3 +685,9 @@ QSize QwtThermo::minimumSizeHint() const
     }
     return QSize( w, h );
 }
+
+// Local Variables:
+// mode: C++
+// c-file-style: "stroustrup"
+// indent-tabs-mode: nil
+// End:
